/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 2.0b 09/20/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#ifdef HAVE_NETINET_TCP_H
#include <netinet/tcp.h>
#endif

#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif

#include "vtun.h"
#include "lib.h"
#include "auth.h"

void client(struct vtun_host *host)
{
     struct sockaddr_in my_addr,svr_addr;
     struct sigaction sa;
     int s;	

     /* 
      * Clear speed and flags suplied by server. 
      */
     host->spd_in = host->spd_out = 0;
     host->flags &= VTUN_CLNT_MASK;

     memset(&sa,0,sizeof(sa));     
     sa.sa_handler=SIG_IGN;
     sigaction(SIGHUP,&sa,NULL);
     sigaction(SIGINT,&sa,NULL);
     sigaction(SIGQUIT,&sa,NULL);
     sigaction(SIGCHLD,&sa,NULL);
     sigaction(SIGPIPE,&sa,NULL);

     memset(&my_addr,0,sizeof(my_addr));
     my_addr.sin_family = AF_INET;
     my_addr.sin_addr.s_addr = vtun.laddr;
     my_addr.sin_port = 0;	

     memset(&svr_addr,0,sizeof(my_addr));
     svr_addr.sin_family = AF_INET;
     svr_addr.sin_addr.s_addr = vtun.saddr;
     svr_addr.sin_port = htons(vtun.port);	

     while(1){ 
	/* We have to create socket again each time
	 * we want to connect, since STREAM sockets 
	 * can be successfuly connected only once.
	 */
	
        if( (s=socket(AF_INET,SOCK_STREAM,0))==-1 ){
	   syslog(LOG_ERR,"Can not create socket");
	   exit(1);
        }

        if( bind(s,(struct sockaddr *)&my_addr,sizeof(my_addr)) ){
	   syslog(LOG_ERR,"Can not bind to the socket");
	   exit(1);
        }

	set_title("connecting to %s",vtun.sname);

        if( connect_t(s,(struct sockaddr *) &svr_addr, vtun.timeout) ){
	   syslog(LOG_INFO,"Can't connect to %s",vtun.sname);
        } else {
	   if( auth_client(s, host) ){   
	      syslog(LOG_INFO,"Session %s[%s] opened",host->host,vtun.sname);

 	      host->rmt_fd = s;
	      start_tunnel(host);
	      /* rmt_fd might be changed in case of UDP proto */	
 	      s = host->rmt_fd;

	      syslog(LOG_INFO,"Session %s[%s] closed",host->host,vtun.sname);
	   } else {
	      syslog(LOG_INFO,"Connection dennied by %s",vtun.sname);
	   }
	}

	close(s);
	
	/* If persist option is set, sleep and try to reconnect */
	if( vtun.persist > 0 ) 
	  sleep(5);
        else
	  break;
     }
     syslog(LOG_INFO,"Exit");
     return;
}
